/*------------------------------------------------------------------------
  Drive a TLC5940 using a Tiny85
  
  This code is (C) 2012 by FTB, all rights reserved.
  ("fungus" on the Arduino forums),
  
  This code is free for non-commercial use (drop me a
  line and let me know how you used it...)

  For commercial use, get in touch via the forums.
------------------------------------------------------------------------

        Tiny85                                        TLC5940
       ----U----                                      ---U----
      |1       8|-> VCC (+5V)                   LED1 |1     28| LED0
      |2       7|-> SCLK, GSCLK (pin 25,18)     LED2 |2     27|-> GND (VPRG)
      |3       6|-> SIN (pin 26)                LED3 |3     26|-> SIN (Tiny85 pin 6)
 GND<-|4       5|-> BLANK, XLAT (pin 23,24)     LED4 |4     25|-> SCLK (Tiny85 pin 7)
       ---------                                  .  |5     24|-> XLAT (Tiny85 pin 5)
                                                  .  |6     23|-> BLANK (Tiny85 pin 5)
                                                  .  |7     22|-> GND
                                                  .  |8     21|-> VCC (+5V)
                                                  .  |9     20|-> 2K resistor -> GND
                                                  .  |10    19|-> +5V (DCPRG)
                                                  .  |11    18|-> GSCLK (Tiny85 pin 7)
                                                  .  |12    17|-> SOUT
                                                  .  |13    16|-> XERR
                                                LED14|14    15| LED15
                                                      --------

    -  Connect the longer leg (anode) of the LEDs to +5V and the shorter leg
       (cathode) to LED(0-15).
    -  The 2K resistor between TLC pin 20 and GND will let ~20mA through each
       LED.  To be precise, it's I = 39.06 / R (in ohms).
 
Greets
------
Thanks to Alex Leone for writing the Arduino TLC5940 library and
providing the excellent schematic above.

*/

#include <Arduino.h>
#include "Tiny85_TLC5940.h"

/*------------------------------------------------------------------------
  Example: A "Knight Rider" effect...
------------------------------------------------------------------------*/
int leds[16];
byte index, ticks;
char direction;

// nb. Overall LED brightness is related to the speed of this function.
// Make it as fast as possible!
void updateLEDs()
{
  // Fade the LEDs and update the TLC5940 PWM values
  for (byte i=0; i<15; ++i) {
    int& d = leds[i];
    tlc5940.set(i,d);    // Set PWM value for LED i
    d -= 16;
    if (d < 0) {
      d = 0;
    }
  }
  // Strobe left/right lighting up 'led[index]'
  if (++ticks == 32) {
    ticks = 0;
    index += direction;
    if (index == 0)  { direction =  1; }
    if (index == 15) { direction = -1; }
    leds[index] = 4095;  // Max brightness
  }
}

/*------------------------------------------------------------------------
  "setup()"
------------------------------------------------------------------------*/
void setup()
{
  // Stop the hardware timers (because interrupts cause flicker!)
  //
  // nb. Functions like 'millis()' and 'delay()' won't work after this.
  // See comments in 'loop()' for how to do timing().
  //
  // Remove these lines only if you know what you're doing,
  // I'm not assuming any responsibility....
  TCCR0A = TCCR0B = 0;
  TCCR1 = 0;
  
  // Initialize the TLC5940 driver
  tlc5940.init();
}

/*------------------------------------------------------------------------
  "loop()"
------------------------------------------------------------------------*/
void loop()
{
  // "tlc5940.update()" takes about 1 millisecond to execute using the
  // Tiny85's internal 8mHz clock, ie. "loop()" will run about 1000
  // times per second. You can use that for timing in your program.
  tlc5940.update();

  // Update all the LEDs here
  updateLEDs();
}


